///
/// Copyright (c) 2016 Dropbox, Inc. All rights reserved.
///
/// Auto-generated by Stone, do not modify.
///

#import <Foundation/Foundation.h>

#import "DBFILESCommitInfo.h"
#import "DBSerializableProtocol.h"

@class DBFILEPROPERTIESPropertyGroup;
@class DBFILESUploadArg;
@class DBFILESWriteMode;

NS_ASSUME_NONNULL_BEGIN

#pragma mark - API Object

///
/// The `UploadArg` struct.
///
/// This class implements the `DBSerializable` protocol (serialize and
/// deserialize instance methods), which is required for all Obj-C SDK API route
/// objects.
///
@interface DBFILESUploadArg : DBFILESCommitInfo <DBSerializable, NSCopying>

#pragma mark - Instance fields

/// A hash of the file content uploaded in this call. If provided and the
/// uploaded content does not match this hash, an error will be returned. For
/// more information see our Content hash
/// https://www.dropbox.com/developers/reference/content-hash page.
@property (nonatomic, readonly, copy, nullable) NSString *contentHash;

#pragma mark - Constructors

///
/// Full constructor for the struct (exposes all instance variables).
///
/// @param path Path in the user's Dropbox to save the file.
/// @param mode Selects what to do if the file already exists.
/// @param autorename If there's a conflict, as determined by mode, have the
/// Dropbox server try to autorename the file to avoid conflict.
/// @param clientModified The value to store as the clientModified timestamp.
/// Dropbox automatically records the time at which the file was written to the
/// Dropbox servers. It can also record an additional timestamp, provided by
/// Dropbox desktop clients, mobile clients, and API apps of when the file was
/// actually created or modified.
/// @param mute Normally, users are made aware of any file modifications in
/// their Dropbox account via notifications in the client software. If true,
/// this tells the clients that this modification shouldn't result in a user
/// notification.
/// @param propertyGroups List of custom properties to add to file.
/// @param strictConflict Be more strict about how each WriteMode detects
/// conflict. For example, always return a conflict error when mode = `update`
/// in `DBFILESWriteMode` and the given "rev" doesn't match the existing file's
/// "rev", even if the existing file has been deleted. This also forces a
/// conflict even when the target path refers to a file with identical contents.
/// @param contentHash A hash of the file content uploaded in this call. If
/// provided and the uploaded content does not match this hash, an error will be
/// returned. For more information see our Content hash
/// https://www.dropbox.com/developers/reference/content-hash page.
///
/// @return An initialized instance.
///
- (instancetype)initWithPath:(NSString *)path
                        mode:(nullable DBFILESWriteMode *)mode
                  autorename:(nullable NSNumber *)autorename
              clientModified:(nullable NSDate *)clientModified
                        mute:(nullable NSNumber *)mute
              propertyGroups:(nullable NSArray<DBFILEPROPERTIESPropertyGroup *> *)propertyGroups
              strictConflict:(nullable NSNumber *)strictConflict
                 contentHash:(nullable NSString *)contentHash;

///
/// Convenience constructor (exposes only non-nullable instance variables with
/// no default value).
///
/// @param path Path in the user's Dropbox to save the file.
///
/// @return An initialized instance.
///
- (instancetype)initWithPath:(NSString *)path;

@end

#pragma mark - Serializer Object

///
/// The serialization class for the `UploadArg` struct.
///
@interface DBFILESUploadArgSerializer : NSObject

///
/// Serializes `DBFILESUploadArg` instances.
///
/// @param instance An instance of the `DBFILESUploadArg` API object.
///
/// @return A json-compatible dictionary representation of the
/// `DBFILESUploadArg` API object.
///
+ (nullable NSDictionary<NSString *, id> *)serialize:(DBFILESUploadArg *)instance;

///
/// Deserializes `DBFILESUploadArg` instances.
///
/// @param dict A json-compatible dictionary representation of the
/// `DBFILESUploadArg` API object.
///
/// @return An instantiation of the `DBFILESUploadArg` object.
///
+ (DBFILESUploadArg *)deserialize:(NSDictionary<NSString *, id> *)dict;

@end

NS_ASSUME_NONNULL_END
